@@ -212,10 +212,6 @@ int igraph_i_lazy_adjlist_init(const igraph_t *graph,
     IGRAPH_ERROR("Cannot create lazy adjlist view", IGRAPH_ENOMEM);
   }
 
-  if (simplify == IGRAPH_I_SIMPLIFY) {
-    IGRAPH_CHECK(igraph_vector_init(&al->svect, al->length));
-  }
-  
   return 0;
 }
 
@@ -227,9 +223,6 @@ void igraph_i_lazy_adjlist_destroy(igraph_i_lazy_adjlist_t *al) {
       Free(al->adjs[i]);
     }
   }
-  if (al->simplify == IGRAPH_I_SIMPLIFY) {
-    igraph_vector_destroy(&al->svect);
-  }
 }
 
 igraph_vector_t *igraph_i_lazy_adjlist_get_real(igraph_i_lazy_adjlist_t *al,
@@ -252,23 +245,8 @@ igraph_vector_t *igraph_i_lazy_adjlist_get_real(igraph_i_lazy_adjlist_t *al,
     }
 
     if (al->simplify == IGRAPH_I_SIMPLIFY) {
-      long int j, n=igraph_vector_size(al->adjs[no]);
-      VECTOR(al->svect)[no] = no+1;
-      for (j=0; j<n; /* nothing */) {
-	long int e=VECTOR(*al->adjs[no])[j];
-	if (VECTOR(al->svect)[e] != no+1) {
-	  VECTOR(al->svect)[e]=no+1;
-	  j++;
-	} else {
-	  VECTOR(*al->adjs[no])[j] = igraph_vector_tail(al->adjs[no]);
-	  igraph_vector_pop_back(al->adjs[no]);
-	  n--;
-	}
-      }
-    } else if (al->simplify == IGRAPH_I_SORT_SIMPLIFY) {
       igraph_vector_t *v=al->adjs[no];
       long int i, p=0, n=igraph_vector_size(v);
-      igraph_vector_sort(v);
       for (i=0; i<n; i++) {
 	if (VECTOR(*v)[i] != no && 
 	    (i==n-1 || VECTOR(*v)[i+1] != VECTOR(*v)[i])) {
@@ -282,3 +260,58 @@ igraph_vector_t *igraph_i_lazy_adjlist_get_real(igraph_i_lazy_adjlist_t *al,
   
   return al->adjs[no];
 }
+
+int igraph_i_lazy_adjedgelist_init(const igraph_t *graph,
+				   igraph_i_lazy_adjedgelist_t *al,
+				   igraph_neimode_t mode) {
+
+  if (mode != IGRAPH_IN && mode != IGRAPH_OUT && mode != IGRAPH_ALL) {
+    IGRAPH_ERROR("Cannot create adjlist view", IGRAPH_EINVMODE);
+  }
+  
+  if (!igraph_is_directed(graph)) { mode=IGRAPH_ALL; }
+  
+  al->mode=mode;
+  al->graph=graph;
+  
+  al->length=igraph_vcount(graph);
+  al->adjs=Calloc(al->length, igraph_vector_t*);
+  if (al->adjs == 0) {
+    IGRAPH_ERROR("Cannot create lazy adjedgelist view", IGRAPH_ENOMEM);
+  }
+
+  return 0;
+  
+}
+
+void igraph_i_lazy_adjedgelist_destroy(igraph_i_lazy_adjedgelist_t *al) {
+  long int i, n=al->length;
+  for (i=0; i<n; i++) {
+    if (al->adjs[i] != 0) {
+      igraph_vector_destroy(al->adjs[i]);
+      Free(al->adjs[i]);
+    }
+  }
+}
+
+igraph_vector_t *igraph_i_lazy_adjedgelist_get_real(igraph_i_lazy_adjedgelist_t *al,
+						    igraph_integer_t pno) {
+  long int no=pno;
+  int ret;
+  if (al->adjs[no] == 0) {
+    al->adjs[no] = Calloc(1, igraph_vector_t);
+    if (al->adjs[no] == 0) {
+      igraph_error("Lazy adjedgelist failed", __FILE__, __LINE__, 
+		   IGRAPH_ENOMEM);
+    }
+    ret=igraph_vector_init(al->adjs[no], 0);
+    if (ret != 0) {
+      igraph_error("", __FILE__, __LINE__, ret);
+    }
+    ret=igraph_adjacent(al->graph, al->adjs[no], no, al->mode);
+    if (ret != 0) {
+      igraph_error("", __FILE__, __LINE__, ret);
+    }
+  }
+  return al->adjs[no];
+}
